"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const { __ΩClassType } = require('@deepkit/core');
const { __ΩIGridRequestDto } = require('./GridRequestDto');
const { __ΩIGridRequestDtoFilter } = require('./GridRequestDto');
const __ΩRecord = ['K', 'T', 'l\'e#"Rb!b"Pde"!N#!'];
const orm_1 = require("@deepkit/orm");
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
class AGrid {
    db;
    constructor(db) {
        this.db = db;
    }
    filterableColumns = null;
    sortableColumns = null;
    searchableColumns = null;
    async filter(dto) {
        let query = this.searchQuery(this.db.query(this.entity));
        const filters = this.addFilter(dto);
        if (!AGrid.isFilterEmpty(filters)) {
            query = query.filter(filters);
        }
        const count = await query.count();
        query = this.addSorter(query, dto);
        query = this.addPaging(query, dto);
        const result = await query.find();
        return new GridResponse_1.default(result, count, dto);
    }
    searchQuery(query) {
        return query;
    }
    addPaging(query, dto) {
        const paging = dto.paging || {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        return query
            .skip((paging.page - 1) * paging.itemsPerPage)
            .limit(paging.itemsPerPage);
    }
    addSorter(query, dto) {
        const allSorts = JSON.parse(JSON.stringify(dto.sorter || []));
        if (dto.additionalSorter) {
            allSorts.push(...dto.additionalSorter);
        }
        if (allSorts.length <= 0) {
            return query;
        }
        let sorted = query;
        allSorts.forEach(Object.assign(sorter => {
            let column = sorter.column;
            if (this.sortableColumns != null) {
                if (column in this.sortableColumns) {
                    column = this.sortableColumns[column];
                }
                else {
                    throw new Error(`[${column}] is not allowed for sorting`);
                }
            }
            sorted = sorted.orderBy(column, sorter.direction.toLowerCase());
        }, { __type: ['sorter', '', 'P"2!"/"'] }));
        return sorted;
    }
    addFilter(dto) {
        const filter = { $and: [] };
        const allFilters = JSON.parse(JSON.stringify(dto.filter || []));
        if (dto.additionalFilter) {
            allFilters.push(...dto.additionalFilter);
        }
        if (allFilters.length > 0) {
            allFilters.forEach(Object.assign(and => {
                const orFilter = [];
                and.forEach(Object.assign(or => {
                    const column = this.getFilterableColumn(or.column);
                    orFilter.push({ [column]: AGrid.createExpression(or) });
                }, { __type: ['or', '', 'P"2!"/"'] }));
                filter.$and.push({ $or: orFilter });
            }, { __type: ['and', '', 'P"2!"/"'] }));
        }
        if (dto.search) {
            const searches = [];
            this.searchableColumns?.forEach(Object.assign(column => {
                const name = this.getFilterableColumn(column);
                searches.push({ [name]: { $regex: dto.search } });
            }, { __type: ['column', '', 'P"2!"/"'] }));
            filter.$and.push({ $or: searches });
        }
        return filter;
    }
    getFilterableColumn(name) {
        let column = name;
        if (this.filterableColumns != null) {
            if (column in this.filterableColumns) {
                column = this.filterableColumns[column];
            }
            else {
                throw new Error(`[${column}] is not allowed for filtering`);
            }
        }
        return column;
    }
    static isFilterEmpty(filters) {
        if (!filters) {
            return true;
        }
        if (Object.keys(filters).length === 1 && '$and' in filters) {
            return filters.$and.length <= 0;
        }
        return false;
    }
    static createExpression(filter) {
        switch (filter.operator) {
            case GridRequestDto_1.Operator.EQ:
                return filter.value[0];
            case GridRequestDto_1.Operator.NEQ:
                return { $ne: filter.value[0] };
            case GridRequestDto_1.Operator.IN:
                return { $in: filter.value };
            case GridRequestDto_1.Operator.NIN:
                return { $nin: filter.value };
            case GridRequestDto_1.Operator.GT:
                return { $gt: filter.value[0] };
            case GridRequestDto_1.Operator.LT:
                return { $lt: filter.value[0] };
            case GridRequestDto_1.Operator.GTE:
                return { $gte: filter.value[0] };
            case GridRequestDto_1.Operator.LTE:
                return { $lte: filter.value[0] };
            case GridRequestDto_1.Operator.LIKE:
                return { $regex: filter.value[0] };
            case GridRequestDto_1.Operator.STARTS:
                return { $regex: `^${filter.value[0]}` };
            case GridRequestDto_1.Operator.ENDS:
                return { $regex: `${filter.value[0]}$` };
            case GridRequestDto_1.Operator.NEMPTY:
                return { $ne: null };
            case GridRequestDto_1.Operator.EMPTY:
                return { $eq: null };
            case GridRequestDto_1.Operator.BETWEEN:
                return {
                    $gte: filter.value[0],
                    $lte: filter.value[1],
                };
            case GridRequestDto_1.Operator.NBETWEEN:
                return {
                    $gte: filter.value[1],
                    $lte: filter.value[0],
                };
        }
        return filter.value;
    }
    static __type = ['T', () => orm_1.Database, 'db', 'constructor', () => __ΩClassType, 'entity', () => __ΩRecord, 'filterableColumns', function () { return null; }, () => __ΩRecord, 'sortableColumns', function () { return null; }, 'searchableColumns', function () { return null; }, () => __ΩIGridRequestDto, 'dto', () => GridResponse_1.default, 'filter', () => orm_1.Query, 'query', () => orm_1.Query, 'searchQuery', () => orm_1.Query, () => __ΩIGridRequestDto, () => orm_1.Query, 'addPaging', () => orm_1.Query, () => __ΩIGridRequestDto, () => orm_1.Query, 'addSorter', () => __ΩIGridRequestDto, () => __ΩRecord, 'addFilter', 'name', 'getFilterableColumn', () => __ΩRecord, 'filters', 'isFilterEmpty', () => __ΩIGridRequestDtoFilter, 'createExpression', 'b!PP7"2#;"0$n%3&<=P&&o\'#,J3(<>)P&&o*#,J3+<>,P&F,J3-<>.Pn/20Pe#!71`02PP"7324P"7506<PP"7724n820P"790:;PP"7;24n<20P"7=0>;Pn?20&"o@#0A;P&2B&0C;P&"oD#2E)0F;PnG22"0H;5'];
}
exports.default = AGrid;
