"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.__ΩFilterList = void 0;
const { __ΩIGridRequestDto } = require('./GridRequestDto');
const { __ΩIGridRequestDtoFilter } = require('./GridRequestDto');
const __ΩRecord = ['K', 'T', 'l\'e#"Rb!b"Pde"!N#!'];
const GridRequestDto_1 = require("./GridRequestDto");
const GridResponse_1 = __importDefault(require("./GridResponse"));
const __ΩFilterList = [() => __ΩRecord, () => __ΩRecord, '&&"o"#o!#'];
exports.__ΩFilterList = __ΩFilterList;
class AKnexGrid {
    knex;
    constructor(knex) {
        this.knex = knex;
    }
    filterableColumns = null;
    sortableColumns = null;
    searchableColumns = null;
    async filter(dto) {
        let query = this.searchQuery(this.knex);
        query = this.addFilters(query, dto);
        const countRes = await query.clone().count();
        query = this.addSorters(query, dto);
        query = this.addPaging(query, dto);
        const result = await query;
        const count = countRes[0]?.['count(*)'] ?? 0;
        return new GridResponse_1.default(result.map(this.toObject), count, dto);
    }
    addPaging(query, dto) {
        const paging = dto.paging || {
            page: 1,
            itemsPerPage: 10,
        };
        paging.page = Math.max(paging.page, 1);
        paging.itemsPerPage = Math.max(paging.itemsPerPage, 1);
        dto.paging = paging;
        return query
            .offset((paging.page - 1) * paging.itemsPerPage)
            .limit(paging.itemsPerPage);
    }
    addSorters(query, dto) {
        let q = query;
        const allSorts = JSON.parse(JSON.stringify(dto.sorter || []));
        if (dto.additionalSorter) {
            allSorts.push(...dto.additionalSorter);
        }
        if (allSorts.length <= 0) {
            return q;
        }
        allSorts.forEach(Object.assign(sorter => {
            let column = sorter.column;
            if (this.sortableColumns != null) {
                if (column in this.sortableColumns) {
                    column = this.sortableColumns[column];
                }
                else {
                    throw new Error(`[${column}] is not allowed for sorting`);
                }
            }
            q = q.orderByRaw(column, sorter.direction.toLowerCase());
        }, { __type: ['sorter', '', 'P"2!"/"'] }));
        return q;
    }
    addFilters(query, dto) {
        let q = query;
        const allFilters = JSON.parse(JSON.stringify(dto.filter || []));
        if (dto.additionalFilter) {
            allFilters.push(...dto.additionalFilter);
        }
        const that = this;
        if (allFilters.length > 0) {
            allFilters.forEach(Object.assign(and => {
                q = q.andWhere(function () {
                    let innerQ = this;
                    and.forEach(Object.assign(or => {
                        const column = that.getFilterableColumn(or.column);
                        innerQ = AKnexGrid.addOrExpression(innerQ, { column, value: or.value, operator: or.operator });
                    }, { __type: ['or', '', 'P"2!"/"'] }));
                });
            }, { __type: ['and', '', 'P"2!"/"'] }));
        }
        if (dto.search && (this.searchableColumns?.length ?? 0) > 0) {
            q = q.andWhere(function () {
                let innerQ = this;
                that.searchableColumns?.forEach(Object.assign(sColumn => {
                    const column = that.getFilterableColumn(sColumn);
                    innerQ = AKnexGrid.addOrExpression(innerQ, { column, value: [dto.search ?? ''], operator: GridRequestDto_1.Operator.LIKE });
                }, { __type: ['sColumn', '', 'P"2!"/"'] }));
            });
        }
        return q;
    }
    getFilterableColumn(name) {
        let column = name;
        if (this.filterableColumns != null) {
            if (column in this.filterableColumns) {
                column = this.filterableColumns[column];
            }
            else {
                throw new Error(`[${column}] is not allowed for filtering`);
            }
        }
        return column;
    }
    static addOrExpression(query, filter) {
        switch (filter.operator) {
            case GridRequestDto_1.Operator.EQ:
                return query.orWhere(filter.column, filter.value[0]);
            case GridRequestDto_1.Operator.NEQ:
                return query.orWhereNot(filter.column, filter.value[0]);
            case GridRequestDto_1.Operator.IN:
                return query.orWhereIn(filter.column, filter.value);
            case GridRequestDto_1.Operator.NIN:
                return query.orWhereNotIn(filter.column, filter.value);
            case GridRequestDto_1.Operator.GT:
                return query.orWhere(filter.column, '>', filter.value[0]);
            case GridRequestDto_1.Operator.LT:
                return query.orWhere(filter.column, '<', filter.value[0]);
            case GridRequestDto_1.Operator.GTE:
                return query.orWhere(filter.column, '>=', filter.value[0]);
            case GridRequestDto_1.Operator.LTE:
                return query.orWhere(filter.column, '<=', filter.value[0]);
            case GridRequestDto_1.Operator.LIKE:
                return query.orWhereLike(filter.column, `%${filter.value[0]}%`);
            case GridRequestDto_1.Operator.STARTS:
                return query.orWhereLike(filter.column, `${filter.value[0]}%`);
            case GridRequestDto_1.Operator.ENDS:
                return query.orWhereLike(filter.column, `%${filter.value[0]}`);
            case GridRequestDto_1.Operator.NEMPTY:
                return query.orWhereNotNull(filter.column);
            case GridRequestDto_1.Operator.EMPTY:
                return query.orWhereNull(filter.column);
            case GridRequestDto_1.Operator.BETWEEN:
                return query.orWhereBetween(filter.column, filter.value);
            case GridRequestDto_1.Operator.NBETWEEN:
                return query.orWhereNotBetween(filter.column, filter.value);
        }
        return query.orWhere(filter.column, filter.value[0]);
    }
    static __type = ['T', 'knex', 'constructor', () => __ΩRecord, 'filterableColumns', function () { return null; }, () => __ΩRecord, 'sortableColumns', function () { return null; }, 'searchableColumns', function () { return null; }, 'searchQuery', () => __ΩRecord, 'data', 'toObject', () => __ΩIGridRequestDto, 'dto', () => GridResponse_1.default, 'filter', 'query', () => __ΩIGridRequestDto, 'addPaging', () => __ΩIGridRequestDto, 'addSorters', () => __ΩIGridRequestDto, 'addFilters', 'name', 'getFilterableColumn', () => __ΩIGridRequestDtoFilter, 'addOrExpression', 'b!P"2";"0#P&&o$#,J3%<>&P&&o\'#,J3(<>)P&F,J3*<>+P"2""0,<=P&#o-#2.e"!0/<=Pn021Pe#!72`03P"24n521"06;P"24n721"08;P"24n921"0:;P&2;&0<;P"24n=23"0>;5'];
}
exports.default = AKnexGrid;
